; Compiler: Avrasm2

#include "m8Adef.inc"

;====
.DSEG
;====

    .org 0x0060

OCR1plsL: .Byte 1 ; to set OCR1BL, low
OCR1plsH: .Byte 1 ; to set OCR1BH, high


;---------------------

; for loop, to clear registers
.equ SRAMstr = 0x60             ; up to 0x00FF (160 SRAM bytes)
;#define SRAMall                 ; to clear all 1024 registers

;---------------------

;====
.CSEG
;====

;==========================
; **** SYSTEM constants ***
;==========================

.equ DDRBini0 = 0b00001101      ; ii ii oo i o
.equ PRTBini0 = 0b11110111      ; PP PP LL P H, PB2=1=MOSFET off

; *** inputs *** 1
#define ADJT_In PINB,6          ; p09 

; *** outputs *** 5
; PB2=OC1B PWM ; Fast PWM, 10-bit, ; clk / 64, 122.070 Hz
; PB5= Dgt3, PB4= Dgt2, PB0= Dgt1

; *** reserved *** 2
; PB1, PB7

;======

.equ DDRCini0  = 0b00011000     ; xi i o o i i i
.equ PRTCini0  = 0b01100101     ; xP P L L P T P

; *** inputs *** 2
#define USER_In PINC,5          ;p28
; PC1= ADC_V

; *** outputs *** 1
#define BUZZ_Out PORTC,4        ; p27
#define TIMEcalb PORTC,3        ; p26

; *** reserved *** 4+1[N/A]
; PC0, PC2, PC4, PC6 (reset), PC7 (N/A)

;======

.equ DDRDini0 = 0b10000000      ; oiiiiiii
.equ PRTDini0 = 0b00000000      ; LTTTTTTT

; *** inputs *** 0
; none

; *** outputs *** 8
.equ Seg_dot = 0b00100000

; *** reserved *** 0
; none

;==========================
; *** PROGRAM constants ***
;==========================

; Fast PWM, Compare Output Mode
; Fast PWM, 10-bit
; OC1A disconnected, Normal port operation.
; Set OC1B on Compare Match, clear OC1B at BOTTOM, (inverting mode)
; clk / 64
.equ TCCR1Aini = 0b00110011
.equ TCCR1Bini = 0b00001011

;=============================
; *** PROGRAM hi-registers ***
;=============================
; specific hi-registers
.def R1flags = r25   ; flags 1
.def R2flags = r24   ; flags 2
.def R3flags = r23   ; flags 3

; division of 16-bit register by 100
; D16c100: [rounding result is always right]
; 0xYYYY  = r17:r16
; result  = r21:r20
; used: r19:r18 , r14:r13

; Division 24/16, 414 cycles, no rounding ***
; D16u16rd: , D24u16u: or D24u16rd:
; also DEC2BCD:, M_16x16:
.def steps = r21      ; loop counter
.def dv_H  = r20      ; divisor, high
.def dv_L  = r19      ; divisor, low
.def dd_U  = r18      ; divident, upper [ destroyed ]
.def dd_H  = r17      ; divident, high  [ destroyed ]
.def dd_L  = r16      ; divident, low   [ destroyed ]

.def rmd_H = r14      ; remainder, high
.def rmd_L = r13      ; remainder, low
.def res_U = r2       ; result, upper
.def res_H = r1       ; result, high
.def res_L = r0       ; result, low

; GLOBAL  hi-registers
;.def tmpHiQ   = r19   ; tempprary register
;.def tmpHiU   = r18   ; tempprary register
;.def tmpHiH   = r17   ; tempprary register
;.def tmpHiL   = r16   ; tempprary register

;=============================
; *** PROGRAM lo-registers ***
;=============================
; GLOBAL  lo-registers
.def zero     = r15   ; for 0 value
;.def tmpLoH   = r14   ; temporary for program
;.def tmpLoL   = r13   ; temporary for program

; specific lo-registers
.def keyDlyH  = r12   ; BOOTdly/2,KEYsavH\L delays
.def keyDlyL  = r11   ;

.def avgCt_H  = r10   ; average Amp counter
.def avgCt_L  = r9    ;

.def curSmpl  = r8    ; for IJMP and find average

.def digtN    = r7    ; current display digit
.def dspOff   = r6    ; dspOff=0, offtime=256*16*104us=426ms

; r2 & r1 & r0 for MUL and others

.equ REGstr = 1
.equ REGend = 25                ; for loop to clear registers

;======================
; *** PROGRAM flags ***
;======================
; Bx for SBRC, SBRS
; Fx for SBR, CBR

;-------------------
;.def R1flags  = r25            ; general flags 1, basic
;-------------------

#define B_24Vmod r25,6          ; 1 = 24V battery detected [avgV18V]
#define F_24Vmod r25,0x40

#define B_newVlt r25,5          ; 1 = ADC voltage reading ready
#define F_newVlt r25,0x20

#define B_ftlErr r25,4          ; 1 = for on-off
#define F_ftlErr r25,0x10

#define B_UsrHlt r25,3          ; 1 = MOS was halted by user
#define F_UsrHlt r25,0x08

#define B_vltMod r25,2          ; 1 = volt mode
#define F_vltMod r25,0x04

#define B_dspOff r25,1          ; 1 = temporary display off
#define F_dspOff r25,0x02
#define X_dspOff 0b00000010

#define B_adjMod r25,0          ; 1 = adjusting test display
#define F_adjMod r25,0x01

;-------------------
;.def R2flags  = r24            ; general flags 2, basic
;-------------------

#define B_do_Pls r24,5          ; 1 = do SET_pwm:
#define F_do_Pls r24,0x20

#define B_vltMax r24,4          ; 1 = very high volt [16V,32V], display on-off
#define F_vltMax r24,0x10

#define B_usrEnd r24,3          ; 1 = wait user to press cancel key 
#define F_usrEnd r24,0x08

#define B_pwmMod r24,2          ; 1 = display PWM duty cycle
#define F_pwmMod r24,0x04

#define B_buzzOn r24,1          ; 1 = turn on buzzer if display off
#define F_buzzOn r24,0x02

#define B_999pwm r24,0          ; 1 = pwm is full on [999]
#define F_999pwm r24,0x01

;-------------------
;.def R3flags  = r23            ; KEYSprs: flags 3
;-------------------

#define B_kyHiOn r23,7          ; 1 = key_hi was pressed
#define F_kyHiOn r23,0x80

#define B_usrVhi r23,6          ; 1 = user prog mode for high voltage
#define F_usrVhi r23,0x40

#define B_shwLmt r23,5          ; 1 = show low limit at boot, for SHWdspl:
#define F_shwLmt r23,0x20

#define B_fltChg r23,4          ; 1 = float charge state
#define F_fltChg r23,0x10

#define B_vltDwn r23,3          ; 1 = battery voltage is decreasing
#define F_vltDwn r23,0x08

;===========================
;===========================

;-------------------
.equ UBRRLs1 = 0x09        ; flags using UBRRL
;-------------------

#define S_Gv_Set 0x09,5    ; 1 = set voltage gain before saving

#define S_Gv_Sav 0x09,4    ; 1 = save voltage gain
#define S_GvSavH 0x09,3    ; 1 = save voltage gain, high

#define S_vHiSav 0x09,2    ; 1= save user high voltage
#define S_vHiSvH 0x09,1    ; 1= save user high voltage, high

;========================================================================

    .org 0x0000

; MCUCSR: * , * , * , * , WDRF , BORF , EXTRF , PORF
    rjmp RESET      ; Reset Handler

    rjmp EXT_INT0   ; IRQ0 Handler
    rjmp EXT_INT1   ; IRQ1 Handler
    rjmp TIM2_COMP  ; Timer2 Compare Handler
    rjmp TIM2_OVF   ; Timer2 Overflow Handler
    rjmp TIM1_CAPT  ; Timer1 Capture Handler
    rjmp TIM1_COMPA ; Timer1 CompareA Handler
    rjmp TIM1_COMPB ; Timer1 CompareB Handler
    rjmp TIM1_OVF   ; Timer1 Overflow Handler
    rjmp TIM0_OVF   ; Timer0 Overflow Handler
    rjmp SPI_STC    ; SPI Transfer Complete Handler
    rjmp USART_RXC  ; USART RX Complete Handler
    rjmp USART_UDRE ; UDR Empty Handler
    rjmp USART_TXC  ; USART TX Complete Handler
    rjmp ADC_RDY    ; ADC Conversion Complete Handler
    rjmp EE_RDY     ; EEPROM Ready Handler
    rjmp ANA_COMP   ; Analog Comparator Handler
    rjmp TWSI       ; Two-wire Serial Interface Handler
    rjmp SPM_RDY    ; Store Program Memory Ready Handler

;=============================
; *RI* Interrupts Handlers ***
;=============================

; IRQ0: PD2, pin4 [MCUCR , GICR , GIFR]
EXT_INT0:
    RETI

; IRQ1: PD3, pin5 [MCUCR , GICR , GIFR]
EXT_INT1:
    RETI

; Timer2 Compare: [TCCR2 , ASSR , TCNT2 , OCR2 , TIMSK , TIFR]
TIM2_COMP:
    RETI

; Timer2 Overflow: [TCCR2 , ASSR , TCNT2 , OCR2 , TIMSK , TIFR]
TIM2_OVF:
    RETI

; Timer1 Capture: [TCCR1A/B,TCNT1H/L,OCR1AH/L,OCR1BH/L,ICR1H/L,TIMSK ,TIFR]
TIM1_CAPT:
    RETI

; Timer1 CompareA: [TCCR1A/B,TCNT1H/L,OCR1AH/L,OCR1BH/L,ICR1H/L,TIMSK ,TIFR]
TIM1_COMPA:
    RETI

; Timer1 CompareB: [TCCR1A/B,TCNT1H/L,OCR1AH/L,OCR1BH/L,ICR1H/L,TIMSK ,TIFR]
TIM1_COMPB:
    RETI

; Timer1 Overflow: [TCNT1H/L, TIMSK , TIFR]
TIM1_OVF:
    RETI

; Timer0 Overflow: [TCCR0 , TCNT0]
TIM0_OVF:           ; TCNT0 [if TCCR0=0x00 always]
    RETI

; SPI Transfer Complete: [SPCR , SPSR , SPDR]
SPI_STC:            ;
    RETI

; USART: RX Complete: [UDR , UDR , UBRRL , UCSRA , UCSRB]
USART_RXC:
    RETI

; USART: UDR Empty: [UDR , UDR , UBRRL , UCSRA , UCSRB]
USART_UDRE:
    RETI

; USART: TX Complete: [UDR , UDR , UBRRL , UCSRA , UCSRB]
USART_TXC:
    RETI

; ADC Conversion Complete: [ADCH/L , ADMUX , ADCSRA]
ADC_RDY:            ;
    RETI

; EEPROM Ready: [EEARH/L, EEDR , EECR]
EE_RDY:
    RETI

; Analog Comparator: [SFIOR , ACSR]
ANA_COMP:
    RETI

; Two-wire Serial Interface: [TWAR , TWDR , TWBR , TWCR , TWSR]
TWSI:
    RETI

; Store Program Memory Ready: [SPMCR]
SPM_RDY:
    RETI

;===================
; *** MAIN entry ***
;===================

RESET:
; SPH\L [Stack Pointer] = RAMEND [top of SRAM, 0x045F]
    LDI   r16,  low(RAMEND)
    LDI   r17, high(RAMEND)
    OUT   SPL, r16
    OUT   SPH, r17

; DDRB=DDRBini0
    LDI   r16, DDRBini0
    OUT   DDRB, r16
; PORTB=PRTBini0
    LDI   r17, PRTBini0
    OUT   PORTB, r17

; DDRC=DDRCini0
    LDI   r16, DDRCini0
    OUT   DDRC, r16
; PORTC=PRTCini0
    LDI   r17, PRTCini0
    OUT   PORTC, r17

; DDRD=DDRDini0
    LDI   r16, DDRDini0
    OUT   DDRD, r16
; PORTD=PRTDini0
    LDI   r17, PRTDini0
    OUT   PORTD, r17

; Wait for stable Vcc (the fuse BODEN should be enabled)
    IN    r16, MCUCSR           ; MCU Control and Status Register
    SBRS  r16, BORF             ; Test Brown-out Reset Flag
    RJMP  RST_cnt               ; Continue if 0

    CLR   r16                   ; Brown-out Reset Flag is 1
    OUT   MCUCSR, r16           ; Clear all flags
    RJMP  RESET                 ;  and reboot

;====================================================================
; clear low and high registers
RST_cnt:
    CLR   r0
    LDI   XH, high(REGstr)
    LDI   XL,  low(REGstr)      ; start of variables in low registers
CLR_lp1:
    ST    X+, r0
    CPI   XL, REGend+1          ; end of variables in high registers
    BREQ  CLRend1

    RJMP  CLR_lp1

CLRend1:
; clear extended static RAM registers
    LDI   XH, high(SRAMstr)
    LDI   XL,  low(SRAMstr)     ; 0x60
CLR_lp2:
    ST    X+, r0

#ifndef SRAMall
    TST   XL
    BREQ  CLRend2               ; 0x60 =< XL =< 0xFF (160)

    RJMP  CLR_lp2
#else ; no SRAMall
    CPI   XH, high(RAMEND+1)    ; 0x04 , RAMEND+1 = 0x0460
    BRLO  CLR_lp2
    CPI   XL,  low(RAMEND+1)    ; 0x60
    BRLO  CLR_lp2
#endif ; SRAMall


CLRend2:
    OUT   UBRRLs1, r0           ; r0 already cleared
;    OUT   TWARs2, r0

;=========================================
; ============ PWM test code =============
;=========================================

; 10-bit Fast PWM, Compare Output Mode
; OC1A disconnected, Normal port operation.
; Set OC1B on Compare Match, clear OC1B at BOTTOM, (inverting mode)
; MCU clk / 64 , 0.125 us * 64 = 8 us
    LDI   r19, TCCR1Aini
    OUT   TCCR1A, r19
    LDI   r19, TCCR1Bini
    OUT   TCCR1B, r19

    LDI   r16,  low(1023)
    LDI   r17, high(1023)
;    LDI   r16,  low(1024)
;    LDI   r17, high(1024)
    STS   OCR1plsH, r17
    STS   OCR1plsL, r16
    OUT   OCR1BH, r17
    OUT   OCR1BL, r16

; Case_1, OCR1BH\L = 1023 (10-bit maximum)
; In real, OC1B (PB2) is low always.
; Here, a high pulse (8us) occurs at the end of the PWM period

; Case2, OCR1BH\L = 1024 (10-bit maximum + 1)
; In real, OC1B (PB2) is high always [OCR1BH\L= 1024 & 0x03FF]
; Here, OC1B (PB2) is low always.
 
Loop1:
    SBIC  PINB,2
    RJMP  END1

    NOP
    RJMP  Loop1

END1:
; OC1B becomes high
    NOP
    RJMP  Loop1

.exit

;=======================================
